@@ -1,5 +1,8 @@
 # %% imports
-from autogaita.resources.utils import write_issues_to_textfile
+from autogaita.resources.utils import (
+    write_issues_to_textfile,
+    standardise_primary_joint_coordinates,
+)
 from autogaita.common2D.common2D_constants import FILE_ID_STRING_ADDITIONS
 import os
 import shutil
@@ -48,6 +51,7 @@ def some_prep(tracking_software, info, folderinfo, cfg):
     analyse_average_x = cfg["analyse_average_x"]
     standardise_x_coordinates = cfg["standardise_x_coordinates"]
     standardise_y_to_a_joint = cfg["standardise_y_to_a_joint"]
+    coordinate_standardisation_xls = cfg["coordinate_standardisation_xls"]
 
     # .............................  move data  ........................................
     # => slightly different for DLC or SLEAP (see local functions below)
@@ -198,6 +202,7 @@ def some_prep(tracking_software, info, folderinfo, cfg):
         "x_standardisation_joint": x_standardisation_joint,
         "standardise_y_to_a_joint": standardise_y_to_a_joint,
         "y_standardisation_joint": y_standardisation_joint,
+        "coordinate_standardisation_xls": coordinate_standardisation_xls,
         "hind_joints": hind_joints,
         "fore_joints": fore_joints,
         "angles": angles,
@@ -236,7 +241,7 @@ def some_prep(tracking_software, info, folderinfo, cfg):
         )
         print(cols_are_weird_message)
         write_issues_to_textfile(cols_are_weird_message, info)
-    # if wanted: fix that deeplabcut inverses y
+    # if wanted: fix that deeplabcut & SLEAP inverse y
     if invert_y_axis:
         for col in data.columns:
             if col.endswith(" y"):
@@ -250,11 +255,6 @@ def some_prep(tracking_software, info, folderinfo, cfg):
         else:
             y_min = data[y_cols].min().min()
         data[y_cols] -= y_min
-    # convert pixels to millimeters
-    if convert_to_mm:
-        for column in data.columns:
-            if not column.endswith("likelihood"):
-                data[column] = data[column] / pixel_to_mm_ratio
     # quick warning if cfg is set to not flip gait direction but to standardise x
     if not flip_gait_direction and standardise_x_coordinates:
         message = (
@@ -295,6 +295,25 @@ def some_prep(tracking_software, info, folderinfo, cfg):
     # reorder the columns we added
     cols = [TIME_COL, "Flipped"]
     data = data[cols + [c for c in data.columns if c not in cols]]
+    # ------------------------------------------------------------------------
+    #                             IMPORTANT
+    # Next two things must be the last that are done in this function, since
+    # joint-standardisation must be done after beam-subtraction & non-beam-height-stand
+    # and pixel-standardisation must be done after joint-standardisation!
+    # ------------------------------------------------------------------------
+    # 1) standardise all primary joint (!) coordinates
+    # => all dimensions are divided by a fixed user-provided value
+    # => note this function has proper errors raised if things should be wrongly
+    #    configured by user!
+    if len(coordinate_standardisation_xls) > 0:
+        data, cfg = standardise_primary_joint_coordinates(
+            data, tracking_software, info, cfg
+        )
+    # 2) convert pixels to millimeters
+    if convert_to_mm:
+        for column in data.columns:
+            if column.endswith((" x", " y")):  # if tuple: endswith any of these
+                data[column] = data[column] / pixel_to_mm_ratio
     return data
 
 
