@@ -4,6 +4,7 @@ from autogaita.group.group_2_data_processing import (
     import_data,
     avg_and_std,
     grand_avg_and_std,
+    load_previous_runs_dataframes,
 )
 from autogaita.group.group_3_PCA import PCA_main
 from autogaita.group.group_4_stats import (
@@ -13,19 +14,11 @@ from autogaita.group.group_4_stats import (
     ANOVA_main,
 )
 from autogaita.group.group_5_plots import plot_results
-from autogaita.group.group_utils import print_start
+from autogaita.group.group_utils import print_start, tukeys_only_info_message
 from autogaita.resources.utils import print_finish, PlotPanel
 import matplotlib
 import matplotlib.pyplot as plt
 
-# %% A note on cross species functionality
-# => This function supports cross species analyses, however the data must be obtained
-#    from the same tracking_software (=> I.e., resulting from autogaita_dlc or _universal3D)
-# => Also users have to ensure that the comparison makes sense across species w.r.t.
-#    features
-# => Adding "both" leg functionality in compute_avg & g_avg_dfs at some point.
-
-
 # .................................  constants  ........................................
 # SET PLT BACKEND
 # Agg is a non-interactive backend for plotting that can only write to files
@@ -62,23 +55,34 @@ def group(folderinfo, cfg):
         # build loading screen
         plot_panel_instance.build_plot_panel_loading_screen()
 
-    # ..................................  unpack  ......................................
+    # ................................  preparation  ...................................
+    # => either creates and sanity-checks folderinfo & cfg or loads it from a previous
+    #    run's config.json file (if load_dir)
     folderinfo, cfg = some_prep(folderinfo, cfg)
 
     # ..............................  print start  ....................................
     # => print start after some_prep since we do some stuff to cfg["PCA_bins"] there
     print_start(folderinfo, cfg)
 
-    # ...................................  import  .....................................
-    # => DLC ONLY: dfs is x-standardised automatically if 1st-level standardised x
-    #   -- As a result all average & std dfs are x-standardised as well
-    dfs, raw_dfs, cfg = import_data(folderinfo, cfg)
+    # .....................  import & transform (a) or load (b)  .......................
 
-    # .................................  avgs & stds  ..................................
-    avg_dfs, std_dfs = avg_and_std(dfs, folderinfo, cfg)
+    # approach a - import & transform (i.e. no previous results to load from)
+    if not folderinfo["load_dir"]:
+        # in dlc/sleap, dfs is x-standardised automatically if 1st-level standardised x
+        # => As a result all average & std dfs are x-standardised as well
+        dfs, raw_dfs, cfg = import_data(folderinfo, cfg)
+        avg_dfs, std_dfs = avg_and_std(dfs, folderinfo, cfg)
+        g_avg_dfs, g_std_dfs = grand_avg_and_std(avg_dfs, folderinfo, cfg)
 
-    # ..............................  grand avgs & stds  ...............................
-    g_avg_dfs, g_std_dfs = grand_avg_and_std(avg_dfs, folderinfo, cfg)
+    # approach b - load previous run's dfs (avg, g_avg & g_std_dfs)
+    else:
+        try:
+            # creating cfg["bin_num"] here too (since done in import_data in (a)!)
+            avg_dfs, g_avg_dfs, g_std_dfs, cfg = load_previous_runs_dataframes(
+                folderinfo, cfg
+            )
+        except (FileNotFoundError, ValueError):
+            return
 
     # ...................................  PCA  ........................................
     if cfg["PCA_variables"]:  # empty lists are falsey!
@@ -105,17 +109,25 @@ def group(folderinfo, cfg):
 
         # ..................................  ANOVA  ...................................
         if cfg["do_anova"]:  # indentation since we check for stats-vars here too!
-            if anova_design_sanity_check(stats_df, folderinfo, cfg):
-                for stats_var in cfg["stats_variables"]:
-                    ANOVA_main(
-                        stats_df,
-                        g_avg_dfs,
-                        g_std_dfs,
-                        stats_var,
-                        folderinfo,
-                        cfg,
-                        plot_panel_instance,
-                    )
+            if anova_design_sanity_check(stats_df, folderinfo, cfg) is not True:
+                just_do_tukeys = True
+                tukeys_only_info_message(folderinfo)
+            else:
+                just_do_tukeys = False
+            # this function is called ANOVA_main even though we might just run Tukeys
+            # with it and not exactly an ANOVA as well
+            # => see conditions in sanity check setting just_do_tukeys
+            for stats_var in cfg["stats_variables"]:
+                ANOVA_main(
+                    stats_df,
+                    g_avg_dfs,
+                    g_std_dfs,
+                    stats_var,
+                    just_do_tukeys,
+                    folderinfo,
+                    cfg,
+                    plot_panel_instance,
+                )
         plt.close("all")
 
     # ..................................  plots  .......................................
